package AC;

use strict;

BEGIN {
	print "module AC loaded - EDF AC power and intensity decoding\n";
}

sub new {
	my %rec;
	my %stt;
	my $self  = {
		_verbose => 0,
		_last => 0,
		_data => \%rec,
		_stats => \%stt
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				printf "AC -> %02x -> %s -> %5.1f (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1] if ($x eq 'inten');
				printf "AC -> %02x -> %s -> %5.0f (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1] if ($x eq 'power');
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	my $str = '';

	$min++;
	$min = 0 if ($min == 12);


	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				my $i;
				my $moy = 0;
				for ($i = 0 ; $i < 12 ; $i++) {
					$moy += $s->{"$u"}{"$x"}[$i];
				}
				$s->{"$u"}{"$x"}[$min] = 0;

				$str .= sprintf "AC %02x %s %5.1f %d\n", $u, $x, $$y[0]/$$y[1], $moy if ($x eq 'inten');
				$str .= sprintf "AC %02x %s %5.0f %d\n", $u, $x, $$y[0]/$$y[1], $moy if ($x eq 'power');
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

        return 0 if ($data[0] != 0x00);

	my $nb;
        my $mode = $data[1];

	if ($mode == 0x27) {
                $nb = 14;
        }
        else {
                return 0;
        }

	my $count = 0;
	my $acfile = main::DOMOPATH . '/AC_0.txt';
        if (open FILE, "$acfile") {
		$count = <FILE>;
        	close FILE;
	}
	else {
 		printf "AC *** file $acfile does not exist\n" if ($self->{_verbose});
		return 0;
	}

	my $base = $count - ($count % 65536);
 	printf "AC *** base=$base count=$count\n" if ($self->{_verbose});

        my $pos = 0;
        my $cnt = 10;   # to skip sync
        my $start = 0;  # wait for data after sync
        my $val = 0;
        my $n = 0;
        my $ll = $nb;
        my @q;

        foreach my $d (@data) {
                for (my $bit = 0 ; $bit < 8 ; $bit++) {
                        $b = ($d & 0x80) ? 0x80 : 0;
                        $d <<= 1;
                        if ($cnt == 0) {
                                $cnt = 5;
                                if ($b == 0 & $nb != 1) {
                                        printf "AC *** bit $cnt ($n) error\n" if ($self->{_verbose});
                                        return 0;
                                }
                                push @q, ($val >> 4) if ($start);
                                $val = 0;
                                $start = 1;
                                last if (--$nb == 0);
                        }
                        elsif ($start) {
                                $val >>= 1;
                                $val |= $b;
                        }
                        $cnt--;
                        $n++;
                }
        }

        my $ck;
        if ($q[0] == 0xc) {
                # Teleinfo
                $ck = 11;
        }

        my $chck = 5;
	my $xor = 0;
        for (my $i = 0 ; $i < $ck ; $i++) {
# printf "$i:%02x ", $q[$i];
                $chck += $q[$i];
		$xor ^= $q[$i];
        }
        $chck &= 0xf;
        $xor &= 0xf;

        if ($chck != $q[$ck]) {
                printf "$head AC *** Checksum error ***\n" if ($self->{_verbose});
                return 0;
        }

        if ($xor != 0) {
                printf "$head AC *** Xor error ***\n" if ($self->{_verbose});
                return 0;
        }

	# Check if just received...
	my $tt = time;
	my $last = $self->{_last};
	$self->{_last} = $tt;
	if (($tt - $last) < 10) {
		print "$head Frame already processed\n" if ($self->{_verbose});
		return 1;
	}

        $mode = $q[0];
        my $addr = ($mode << 4) | $q[1];
        $head .= sprintf " AC chck:%x", $chck if ($self->{_verbose});

	my $ok = 0;
        my $p = $self->{_data};
	my $s = $self->{_stats};

	my $power = ($q[5] << 12) + ($q[4] << 8) + ($q[3] << 4) + $q[2]; 
	$power += $base;
	$power += 65536 if ($power < $count);
	printf "$head power: %5d\n", $power if ($self->{_verbose});
	$p->{"01"}{power}[0] = $power;
	$p->{"01"}{power}[1] = 1;

	my $inten = (($q[8] << 8) + ($q[7] << 4) + $q[6]) / 16.0;
	printf "$head inten: %4.1f\n", $inten if ($self->{_verbose});
	$p->{"01"}{inten}[0] += $inten;
	my $val = ++($p->{"01"}{inten}[1]);

	$s->{"01"}{inten}[$min] = $val;
	$s->{"01"}{power}[$min] = $val;

	$ok = 1;

        if (open FILE, ">$acfile") {
		print FILE $power;
        	close FILE;
	}

	return $ok;
}

sub command {
	return 0;
}

END {
#	print "module AC unloaded\n";
}

1;  # so the require or use succeeds

