package EX;

use strict;

BEGIN {
	print "module EX loaded - Electric extended sensors decoding\n";
}

sub new {
	my %rec;
	my $self  = {
		_verbose => 0,
		_data => \%rec
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				printf "EX -> %02x -> %s -> %5d (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $str = '';

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				$str .= sprintf "EX %02x %s %5d %d\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	# return if not for me ...
	return 0 if ($data[0] != 0x00);

	my $mode = $data[1];
	my $addr = $data[2] >> 4;
	my $chck = 0;

	$head .= sprintf "EX mode:%02x addr:%x chck:%x", $mode, $addr, $chck & 0xf 
		if ($self->{_verbose});

	my $nb;

	if ($mode == 0x25 && $addr != 0xe) {
		$nb = 15;
	}
	else {
		return 0;
	}

	my $pos = 0;
	my $cnt = 10;	# to skip sync
	my $start = 0;	# wait for data after sync
 	my $val = 0;
	my @q;

	foreach my $d (@data) {
		for (my $bit = 0 ; $bit < 8 ; $bit++) {
			$b = ($d & 0x80) ? 0x80 : 0;
			$d <<= 1;
			if ($cnt == 0) {
				$cnt = 5;
				if ($b == 0) {
					print "*** bit $cnt error\n" if ($self->{_verbose});
					return 0;
				}
				push @q, ($val >> 4) if ($start);
				$val = 0;
				$start = 1;
				last if (--$nb == 0);
			}
			elsif ($start) {
				$val >>= 1;
				$val |= $b;
			}
			$cnt--;
		}
	}

	my $ok = 0;
	my $p = $self->{_data};
	
	if ($addr == 0x2) {
		my $elec0 = ($q[2] << 12) + ($q[3] << 8) + ($q[4] << 4) + $q[5];
		printf "$head elec0:%5d\n", $elec0 if ($self->{_verbose});
		$p->{"$addr"}{elec0}[0] += $elec0;
		$p->{"$addr"}{elec0}[1]++;
		my $elec1 = ($q[6] << 12) + ($q[7] << 8) + ($q[8] << 4) + $q[9];
		printf "$head elec1:%5d\n", $elec1 if ($self->{_verbose});
		$p->{"$addr"}{elec1}[0] += $elec1;
		$p->{"$addr"}{elec1}[1]++;
		my $el_st = $q[11];
		printf "$head el_st:%5d\n", $el_st if ($self->{_verbose});
		$p->{"$addr"}{el_st}[0] = $el_st;
		$p->{"$addr"}{el_st}[1] = 1;
		$ok = 1;
	}
	elsif ($addr == 0xa) {
		my $elec2 = ($q[2] << 12) + ($q[3] << 8) + ($q[4] << 4) + $q[5];
		printf "$head elec2:%5d\n", $elec2 if ($self->{_verbose});
		$p->{"$addr"}{elec2}[0] += $elec2;
		$p->{"$addr"}{elec2}[1]++;
		my $elec3 = ($q[6] << 12) + ($q[7] << 8) + ($q[8] << 4) + $q[9];
		printf "$head elec3:%5d\n", $elec3 if ($self->{_verbose});
		$p->{"$addr"}{elec3}[0] += $elec3;
		$p->{"$addr"}{elec3}[1]++;
		my $el_st = $q[11];
		printf "$head el_st:%5d\n", $el_st if ($self->{_verbose});
		$p->{"$addr"}{el_st}[0] = $el_st;
		$p->{"$addr"}{el_st}[1] = 1;
		$ok = 1;
	}
	elsif ($addr == 0x6) {
		my $elec4 = ($q[2] << 12) + ($q[3] << 8) + ($q[4] << 4) + $q[5];
		printf "$head elec4:%5d\n", $elec4 if ($self->{_verbose});
		$p->{"$addr"}{elec4}[0] += $elec4;
		$p->{"$addr"}{elec4}[1]++;
		my $elec5 = ($q[6] << 12) + ($q[7] << 8) + ($q[8] << 4) + $q[9];
		printf "$head elec5:%5d\n", $elec5 if ($self->{_verbose});
		$p->{"$addr"}{elec5}[0] += $elec5;
		$p->{"$addr"}{elec5}[1]++;
		my $el_st = $q[11];
		printf "$head el_st:%5d\n", $el_st if ($self->{_verbose});
		$p->{"$addr"}{el_st}[0] = $el_st;
		$p->{"$addr"}{el_st}[1] = 1;
		$ok = 1;
	}
	else {
		# print "\n" if ($self->{_verbose});
	}

	# if ($self->{_verbose}) {
		# printf("%1X ", $_) foreach (@q);
		# print "\n";
	# }

	return $ok;
}

sub command {
        return 0;
}

END {
#	print "module EX unloaded\n";
}

1;  # so the require or use succeeds

