package IO;

use strict;
use POSIX qw( strftime );
use constant TIMEFORMAT => '%d-%b-%y %T';

BEGIN {
	print "module IO loaded - Relay I/O generic board\n";
}

sub new {
	my %rec;
	my $self  = {
		_verbose => 0,
		_data => \%rec
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				if ($u <= 2) {
					printf "IO -> %02x -> %s -> %02X (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
				}
				else {
					printf "IO -> %02x -> %s -> %d (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
				}
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $str = '';

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				if ($u <= 2) {
					$str .= sprintf "IO %02x %s  0x%02X %d\n", $u, $x, $$y[0]/$$y[1], $$y[1];
				}
				else {
					$str .= sprintf "IO %02x %s %5d %d\n", $u, $x, $$y[0]/$$y[1], $$y[1];
				}
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub write_data {
	my ($self) = @_;

        my $t = time;
        my $d = $t . " " . &strftime(TIMEFORMAT, localtime($t)) . "\n";
        my $str = '';
        $str .= $d;
        $str .= get($self);

        open  TMP, ">/var/lib/domo/io.dat.tmp";
        print TMP $str;
        close TMP;

        rename "/var/lib/domo/io.dat.tmp", "/var/lib/domo/io.dat";
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	# return if not for me ...
	return 0 if ($data[0] != 0x03);

	my $nd1 = $data[1];
	my $nd2 = $data[2];
	my $nbRestarts = $data[3];
	my $nbWatchdogs = $data[4];
	my $nbresets = $data[5];

	my $ok = 0;
	my $p = $self->{_data};

	{
		printf "$head IO stat: rl:%02X", $nd1 if ($self->{_verbose});
		$p->{1}{relay}[0] = $nd1;
		$p->{1}{relay}[1] = 1;

		printf " in:%02X", $nd2 if ($self->{_verbose});
		$p->{2}{input}[0] = $nd2;
		$p->{2}{input}[1] = 1;

		printf " st:%d", $nbRestarts if ($self->{_verbose});
		$p->{3}{rstrt}[0] = $nbRestarts;
		$p->{3}{rstrt}[1] = 1;

		printf " wd:%d", $nbWatchdogs if ($self->{_verbose});
		$p->{4}{wtchd}[0] = $nbWatchdogs;
		$p->{4}{wtchd}[1] = 1;

		printf " rs:%d", $nbresets if ($self->{_verbose});
		$p->{5}{reset}[0] = $nbresets;
		$p->{5}{reset}[1] = 1;

		$ok = 1;
	}
	printf "\n" if ($self->{_verbose});

	write_data($self);

	return $ok;
}

sub command {
	my ($self, $t, $data) = @_;

	if (-w "/tmp/IO_cmd") {
		if (open CMD, "/tmp/IO_cmd") {
			$$data[1] = 2;	# IO card has HW address 2
			$$data[3] = 3;	# command is 3 for command
			my $pos = 4;
			my $line = <CMD>;
			chomp $line;
print("command line = <$line>\n");
			my @cmds = split(/ /, $line);
			$$data[$pos++] = $_ foreach (@cmds);
			close CMD;
		}
		unlink("/tmp/IO_cmd");
		return 1;
	}

	return 0;
}

END {
#	print "module IO unloaded\n";
}

1;  # so the require or use succeeds

