package SL;

use strict;

BEGIN {
	print "module SL loaded - Solar pannels parameters processing\n";
}

sub new {
	my %rec;
	my $self  = {
		_verbose => 0,
		_data => \%rec
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				printf "SL -> %02x -> %s -> %5d (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $str = '';

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				$str .= sprintf "SL %02x %s %5d %d\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	# return if not for me ...
	return 0 if ($data[0] != 0x0a);

	my $mode = $data[1] >> 4;
	return 0 if ($mode != 0xa && $mode != 0xb);

	my $addr = 0xfe & (($data[1] & 0xf) << 4 | $data[2] >> 4);
	my $chck = 0x0f & (
			($data[0] >> 4) + ($data[0] & 0xf) +
			($data[1] >> 4) + ($data[1] & 0xf) +
			($data[2] >> 4) + ($data[2] & 0xf) +
			($data[3] >> 4) + ($data[3] & 0xf) +
			($data[4] >> 4) + ($data[4] & 0xf));

	printf "$head SL mode:%x addr:%02x chck:%x ", $mode, $addr, $chck & 0xf 
		if ($self->{_verbose});

	if ($chck != ($data[5] >> 4)) {
		print "*** Checksum error ***\n" if ($self->{_verbose});
		return 0;
	}

	my $ok = 0;
	my $p = $self->{_data};
	
	if ($mode == 0xa) {
		my $sonde =	($data[2] & 0xf) * 10000 +
				($data[3] >> 4) * 1000 +
				($data[3] & 0xf) * 100 +
				($data[4] >> 4) * 10 +
				($data[4] & 0xf);
		printf "sonde:%5d", $sonde if ($self->{_verbose});
		$p->{"$addr"}{sonde}[0] += $sonde;
		$p->{"$addr"}{sonde}[1]++;
		$ok = 1;
	}
	elsif ($mode == 0xb) {
		my $status =	($data[2] & 0xf) * 10000 +
				($data[3] >> 4) * 1000 +
				($data[3] & 0xf) * 100 +
				($data[4] >> 4) * 10 +
				($data[4] & 0xf);
		printf "state:%5d", $status if ($self->{_verbose});
		$p->{"$addr"}{state}[0] = $status;
		$p->{"$addr"}{state}[1] = 1;
		$ok = 1;
	}
	printf "\n" if ($self->{_verbose});

	return $ok;
}

sub command {
        return 0;
}

END {
#	print "module SL unloaded\n";
}

1;  # so the require or use succeeds

