package TX;

use strict;

BEGIN {
	print "module TX loaded - Lacrosse TX type meteo sensors decoding\n";
}

sub new {
	my %rec;
	my %stt;
	my $self  = {
		_verbose => 0,
		_data => \%rec,
		_stats => \%stt
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				printf "TX -> %02x -> %s -> %5.1f (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	my $str = '';

	$min++;
	$min = 0 if ($min == 12);

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				my $i;
				my $nb;
				my $moy = 0;
				for ($i = 0 ; $i < 12 ; $i++) {
					$moy += $s->{"$u"}{"$x"}[$i];
					$nb++ if (exists($s->{"$u"}{"$x"}[$i]));
				}
				$moy = ($moy * 12) / $nb if ($nb > 0);
				$s->{"$u"}{"$x"}[$min] = 0;
				$str .= sprintf "TX %02x %s %5.1f %d\n", $u, $x, $$y[0]/$$y[1], $moy;
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	# return if not for me ...
	return 0 if ($data[0] != 0x0a);

	my $mode = $data[1] >> 4;
	return 0 if ($mode != 0x0 && $mode != 0xe);

	my $addr = 0xfe & (($data[1] & 0xf) << 4 | $data[2] >> 4);
	my $chck = 0x0f & (
			($data[0] >> 4) + ($data[0] & 0xf) +
			($data[1] >> 4) + ($data[1] & 0xf) +
			($data[2] >> 4) + ($data[2] & 0xf) +
			($data[3] >> 4) + ($data[3] & 0xf) +
			($data[4] >> 4) + ($data[4] & 0xf));

	printf "$head TX mode:%x addr:%02x chck:%x ", $mode, $addr, $chck & 0xf 
		if ($self->{_verbose});

	if ($chck != ($data[5] >> 4)) {
		print "*** Checksum error ***\n" if ($self->{_verbose});
		return 0;
	}

	if ((($data[2] & 0xf) != ($data[4] >> 4)) || (($data[3] >> 4) != ($data[4] & 0xf))) {
		print "*** Format error ***\n" if ($self->{_verbose});
		return 0;
	}

	my $ok = 0;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	
	if ($mode == 0x0) {
		my $tempe =	-50.0 + 
				($data[2] & 0xf) * 10 + 
				($data[3] >> 4) + 
				($data[3] & 0xf) * 0.1;
		printf "tempe:%5.1f", $tempe if ($self->{_verbose});
		$p->{"$addr"}{tempe}[0] += $tempe;
		++($p->{"$addr"}{tempe}[1]);
		++($s->{"$addr"}{tempe}[$min]);
		$ok = 1;
	}
	elsif ($mode == 0xe) {
		my $hygro =	($data[2] & 0xf) * 10 + 
				($data[3] >> 4) + 
				($data[3] & 0xf) * 0.1;
		printf "hygro:%5.1f", $hygro if ($self->{_verbose});
		$p->{"$addr"}{hygro}[0] += $hygro;
		++($p->{"$addr"}{hygro}[1]);
		++($s->{"$addr"}{hygro}[$min]);
		$ok = 1;
	}
	printf "\n" if ($self->{_verbose});

	return $ok;
}

sub command {
        return 0;
}

END {
#	print "module TX unloaded\n";
}

1;  # so the require or use succeeds

