package VT;

use POSIX qw( strftime setsid );
use strict;

BEGIN {
	print "module VT loaded - Power voltage from APC UPS\n";
}

sub new {
	my %rec;
	my %stt;
	my $self  = {
		_verbose => 0,
		_last => 0,
		_data => \%rec,
		_stats => \%stt
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				printf "VT -> %02x -> %s -> %5.1f (%d)\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	my $str = '';

	$min++;
	$min = 0 if ($min == 12);

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				my $i;
				my $moy = 0;
				for ($i = 0 ; $i < 12 ; $i++) {
					$moy += $s->{"$u"}{"$x"}[$i];
				}
				$s->{"$u"}{"$x"}[$min] = 0;
				$str .= sprintf "VT %02x %s %5.1f %d\n", $u, $x, $$y[0]/$$y[1], $moy;
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	# Check if just received...
	my $tt = time;
	my $last = $self->{_last};
	return 0 if (($tt - $last) < 60);

	$self->{_last} = $tt;
	my $ok = 0;
        my $p = $self->{_data};
	my $s = $self->{_stats};

	open(UPS, 'apcaccess status |');
	my @upinfo = <UPS>;
	close(UPS);

	my $voltage=0;
	my $val = 0;
	foreach (@upinfo) {
		$voltage = $1 if $_ =~ m/LINEV.*: (.*) Volts/;
	}
	if ($voltage > 0) {
		printf strftime($timeformat, localtime) . " len=0  : " if ($self->{_verbose});
		printf "VT addr:01 volta: %3d\n", $voltage if ($self->{_verbose});
		$p->{"01"}{volta}[0] += $voltage;
		$val = ++($p->{"01"}{volta}[1]);
		$s->{"01"}{volta}[$min] = $val;
		$ok = 1;
	}

	return $ok;
}

sub command {
	return 0;
}

END {
#	print "module VT unloaded\n";
}

1;  # so the require or use succeeds

