package WS;

use strict;
use POSIX qw( strftime setsid );


BEGIN {
	print "module WS loaded - Lacrosse WS type meteo sensors decoding\n";
}

sub new {
	my %rec;
	my %stt;
	my $self  = {
		_verbose => 0,
		_data => \%rec,
		_stats => \%stt
	};
	bless($self);
	return $self;
}

sub show {
	my ($self) = @_;
	my $p = $self->{_data};

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				my $fmt = ($x eq 'pluvi' || $x eq 'direc') ? '%5d' : '%5.1f';
				printf "WS %02x %s $fmt %d\n", $u, $x, $$y[0]/$$y[1], $$y[1];
			}
		}
	}
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub get {
	my ($self, $min) = @_;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	my $str = '';

	while ((my $u, my $v) = each %$p) {
		if ($v) {
			my %w = %$v;
			while ((my $x, my $y) = each %w) {
				my $fmt = ($x eq 'pluvi' || $x eq 'direc') ? '%5d' : '%5.1f';
				my $i;
				my $moy = 0;
				for ($i = 0 ; $i < 12 ; $i++) {
					$moy += $s->{"$u"}{"$x"}[$i];
				}
				$s->{"$u"}{"$x"}[$min] = 0;
				$str .= sprintf "WS %02x %s $fmt %d\n", $u, $x, $$y[0]/$$y[1], $moy;
			}
		}
	}

	# clear the data
	%$p = -1;

	# return info
	return $str;
}

sub process {
	my ($self, $head, $timeformat, $min, @data) = @_;

	my $val = sprintf("%02X", $data[0]);
	# return if not for me ...
	return 0 if ($data[0] != 0x00);

	my $mode = $data[1];
	my $addr = $data[2] >> 4;
	my $nb;

	# if ($mode == 0x25 && $addr == 0xe) {
	if ($mode == 0x25) {
		$nb = 15;
	}
	elsif ($mode == 0x29) {
		$nb = 8;
	}
	elsif ($mode == 0x39) {
		$nb = 11;
	}
	else {
		return 0;
	}

	my $pos = 0;
	my $cnt = 10;	# to skip sync
	my $start = 0;	# wait for data after sync
 	my $val = 0;
	my @q;

	foreach my $d (@data) {
		for (my $bit = 0 ; $bit < 8 ; $bit++) {
			$b = ($d & 0x80) ? 0x80 : 0;
			$d <<= 1;
			if ($cnt == 0) {
				$cnt = 5;
				if ($b == 0) {
					print "*** bit $cnt error\n" if ($self->{_verbose});
					return 0;
				}
				push @q, ($val >> 4) if ($start);
				$val = 0;
				$start = 1;
				last if (--$nb == 0);
			}
			elsif ($start) {
				$val >>= 1;
				$val |= $b;
			}
			$cnt--;
		}
	}

        my $ck;
        if ($q[0] == 2) {
                # WS700-16 (rain)
                $ck = 6;
        }
        elsif ($q[0] == 3) {
                # WS700-13 (anemometer)
                $ck = 9;
        }
        elsif ($q[0] == 4) {
                # WS700-20 (Thermo/hygro/baro)
                $ck = 13;
        }
        elsif ($q[0] == 5) {
                # WS2500-19 (luxmeter)
                $ck = 10;
        }

	my $xor = 0;
        for (my $i = 0 ; $i < $ck ; $i++) {
                $xor ^= $q[$i];
        }
        $xor &= 0xf;

        if ($xor != 0) {
                printf "*** Xor error ***\n" if ($self->{_verbose});
                return 0;
        }

        my $chck = 5;
        for (my $i = 0 ; $i < $ck ; $i++) {
                $chck += $q[$i];
        }
        $chck &= 0xf;

        if ($chck != $q[$ck]) {
                printf "*** Checksum error ***\n" if ($self->{_verbose});
                return 0;
        }

        $mode = $q[0] & 0xf;
        $addr = $q[1] & 0xf;
	my $negat = (($addr & 0x08) != 0);
	$addr &= 0x7;
        $head .= sprintf " WS type:%x addr:%x chck:%x", $mode, $addr, $chck;

	my $ok = 0;
	my $p = $self->{_data};
	my $s = $self->{_stats};
	
	if ($mode == 2) {
		my $pluvio = ($q[4] << 8) + ($q[3] << 4) + $q[2];
		printf "$head pluvi:%5d\n", $pluvio if ($self->{_verbose});
		$p->{"$addr"}{pluvi}[0] = $pluvio;
		$p->{"$addr"}{pluvi}[1] = 1;
		my $val = ++($p->{"$addr"}{pluv_}[1]);
		$s->{"$addr"}{pluvi}[$min] = $val;
		$ok = 1;
	}
        elsif ($mode == 3) {
                my $speed = $q[4] * 10 + $q[3] + $q[2] * 0.1;
                printf "$head speed:%5.1f\n", $speed if ($self->{_verbose});
                $p->{"$addr"}{speed}[0] += $speed;
                $p->{"$addr"}{speed}[1]++;
                my $direc = ($q[7] & 0x3) * 100   + $q[6] * 10 + $q[5];
                printf "$head direc:%3d\n", $direc if ($self->{_verbose});
                $p->{"$addr"}{direc}[0] += $direc;
                my $val = ++($p->{"$addr"}{direc}[1]);
		$s->{"$addr"}{speed}[$min] = $val;
		$s->{"$addr"}{direc}[$min] = $val;
		$ok = 1;
        }
	elsif ($mode == 4) {
		my $tempe = $q[4] * 10 + $q[3] + $q[2] * 0.1;
		$tempe = -$tempe if ($negat);
		printf "$head tempe:%5.1f\n", $tempe if ($self->{_verbose});
		$p->{"$addr"}{tempe}[0] += $tempe;
		$p->{"$addr"}{tempe}[1]++;
		my $hygro = $q[7] * 10 + $q[6] + $q[5] * 0.1;
		printf "$head hygro:%5.1f\n", $hygro if ($self->{_verbose});
		$p->{"$addr"}{hygro}[0] += $hygro;
		$p->{"$addr"}{hygro}[1]++;
		my $barom = $q[10] * 100   + $q[9] * 10 + $q[8];
		$barom += ($q[11] * 0.1) if ($addr == 0x3);
		my $unk = $q[11] & 0xf;
		# printf "$head barom:%3d [%x] [%x] [%x]\n", $barom, $unk, $q[12] & 0xf, $q[13] & 0xf if ($self->{_verbose});
		printf "$head barom:%5.1f\n", $barom if ($self->{_verbose});
		$p->{"$addr"}{barom}[0] += $barom;
		my $val = ++($p->{"$addr"}{barom}[1]);
		$s->{"$addr"}{tempe}[$min] = $val;
		$s->{"$addr"}{hygro}[$min] = $val;
		$s->{"$addr"}{barom}[$min] = $val;
		$ok = 1;
	}
	elsif ($mode == 5) {
                my $expos = ($q[4] * 100 + $q[3] * 10 + $q[2]) * (10 ** $q[5]);
                printf "$head expos:%5d\n", $expos if ($self->{_verbose});
                $p->{"$addr"}{expos}[0] += $expos;
		my $val = ++($p->{"$addr"}{expos}[1]);
                my $sunsh = ($q[8] << 8) + ($q[7] << 4) + $q[6];
                printf "$head sunsh:%5d\n", $sunsh if ($self->{_verbose});
                $p->{"$addr"}{sunsh}[0] = $sunsh;
                $p->{"$addr"}{sunsh}[1] = 1;
		$s->{"$addr"}{expos}[$min] = $val;
		$s->{"$addr"}{sunsh}[$min] = $val;
		$ok = 1;
	}
	else {
		print "$head Unknown mode $mode\n" if ($self->{_verbose});
	}

	return $ok;
}

sub command {
        return 0;
}

END {
#	print "module WS unloaded\n";
}

1;  # so the require or use succeeds

