package updateRRD;

use strict;
use RRDs;
use DBI;
use POSIX qw( strftime );
use constant TIMEFORMAT => '%d-%b-%y %T';

BEGIN {
	print "PlugIns updateRRD loaded - Update the RRD database\n";
}

sub new {
	my %rec;
        our $host;
        our $database;
        our $login;
        our $password;
        our $rrddir;

        do 'domo.conf';

        my $self  = {
                _verbose => 0,
                _data => \%rec,
                _host => $host,
                _database => $database,
                _login => $login,
                _password =>$password,
                _rrddir =>$rrddir
        };

	if (! -d $rrddir) {
		mkdir($rrddir);
		print "Directory $rrddir does not exist... Created\n";
	}

	bless($self);
	return $self;
}

sub name {
        return "updateRRD";
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub process {
	my ($self, $timeformat, $min, $str) = @_;

        # open database
        my $host = $self->{_host};
        my $database = $self->{_database};
        my $login = $self->{_login};
        my $password = $self->{_password};
        my $rrddir = $self->{_rrddir};
	my $verbose = $self->{_verbose};

        my $dsn = "DBI:mysql:database=$database;host=$host";
        my $dbh = DBI->connect($dsn, $login, $password) or die "Echec connexion to database";
	my $nb_rrds = 100;
	my $wr = 0;
	my $dt = "";
	my ($val1, $val2, $val3, $val4, $val5, $val6, $val7, $val8);
	
	# *************************************************************************
	# Update addresses of TX sensors
	# *************************************************************************
	
	my $done;
	my @sensors;

	do {
		$done = 1;

		@sensors = ();
		my @rrds;
	
		my $query = "SELECT * FROM names";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		while (my $rec = $sth->fetchrow_hashref) {
			# print "Proto = " . $rec->{proto} . " Addr = " . $rec->{addr} . " Rrd = " . $rec->{rrd} . " Name = " . $rec->{name} . "\n";
			$rrds[$rec->{rrd}] = 1;
			push @sensors, $rec;
		}

		$sth->finish;
	
		foreach my $sensor (@sensors) {
			if ($sensor->{proto} eq 'EL') {
				if ($sensor->{rrd} == 0) {
					my $rrd = 0;
					$rrd = 1 if ($sensor->{addr} eq '28');
					$rrd = 2 if ($sensor->{addr} eq '2a');
					$rrd = 4 if ($sensor->{addr} eq '2c');
					my $id = $sensor->{id};
					my $query = "UPDATE names set rrd=$rrd WHERE id=$id";
					# print "query=$query\n";
					my $sth = $dbh->prepare($query);
					my $res = $sth->execute;
					$sth->finish;
				}
			}
			elsif ($sensor->{proto} eq 'TX') {
				if ($sensor->{rrd} == 0) {
					my $rrd = 0;
					for(my $i = 1 ; $i <= $nb_rrds ; $i++) {
						if (!defined($rrds[$i])) {
							$rrds[$i] = 1;
							$rrd = $i;
							$done = 0;
							last;
						}
					}
					my $id = $sensor->{id};
					my $query = "UPDATE names set rrd=$rrd WHERE id=$id";
					# print "query=$query\n";
					my $sth = $dbh->prepare($query);
					my $res = $sth->execute;
					$sth->finish;
				}
			}
		}
	} while ($done == 0);
	
	# *************************************************************************
	# TX sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'TX');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='TX' AND addr='$addr' AND DATE_SUB(CURDATE(),INTERVAL 31 MINUTE) <= date";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = $val2 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			# print "rrd=$rrd name=$name\n";
			# $val1{$name} = $row->{val} if ($row->{type} eq 'tempe');
			# $val2{$name} = $row->{val} if ($row->{type} eq 'hygro');
			$val1 = $row->{val} if ($row->{type} eq 'tempe');
			$val2 = $row->{val} if ($row->{type} eq 'hygro');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}

		$sth->finish;
	
		$dt = "N:$val1:$val2";
		&update_rrd($rrddir, "TX_$rrd", $dt, 'TX', $verbose) if ($wr);
	}
	
	# *************************************************************************
	# AC sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'AC');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='AC' AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = $val2 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if ($row->{type} eq 'inten');
			$val2 = $row->{val} if ($row->{type} eq 'power');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}

		$sth->finish;
	
		$dt = "N:$val1:$val2";
		&update_rrd($rrddir, "AC_$rrd", $dt, 'AC', $verbose) if ($wr);
	}
	
	
	
	# *************************************************************************
	# EL sensors
	# *************************************************************************
	
	$wr = 0;
	$val1 = $val2 = $val3 = $val4 = 'U';
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'EL');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='EL' AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if (($row->{type} eq 'inten') && ($addr eq '28'));
			$val2 = $row->{val} if (($row->{type} eq 'inten') && ($addr eq '2a'));
			$val3 = $row->{val} if (($row->{type} eq 'inten') && ($addr eq '2c'));
			$val4 = $row->{val} if (($row->{type} eq 'volta') && ($addr eq '2c'));
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}

		$sth->finish;
	
	}

	if ($wr) {
		$dt = "N:$val1:$val2:$val3:$val4";
		&update_rrd($rrddir, "EL_1", $dt, 'EL', $verbose) if ($wr);
	}
	
	
	
	# *************************************************************************
	# AC/VT sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'VT');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE (proto='VT') AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if ($row->{type} eq 'volta');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}

		$sth->finish;
	
		$dt = "N:$val1";
		&update_rrd($rrddir, "VT_$rrd", $dt, 'VT', $verbose) if ($wr);
	}
	
	
	
	# *************************************************************************
	# EX sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'EX');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='EX' AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = $val2 = $val3 = $val4 = $val5 = $val6 = $val7 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if ($row->{type} eq 'elec0');
			$val2 = $row->{val} if ($row->{type} eq 'elec1');
			$val3 = $row->{val} if ($row->{type} eq 'elec2');
			$val4 = $row->{val} if ($row->{type} eq 'elec3');
			$val5 = $row->{val} if ($row->{type} eq 'elec4');
			$val6 = $row->{val} if ($row->{type} eq 'elec5');
			$val7 = $row->{val} if ($row->{type} eq 'el_st');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}
	
		$sth->finish;

		$dt = "N:$val1:$val2:$val3:$val4:$val5:$val6:$val7";
		&update_rrd($rrddir, "EX_$rrd", $dt, 'EX', $verbose) if ($wr);
	}
	
	
	# *************************************************************************
	# SL sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'SL');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='SL' AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = $val2 = $val3 = $val4 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if ($addr eq '38');
			$val2 = $row->{val} if ($addr eq '3a');
			$val3 = $row->{val} if ($addr eq '3c');
			$val4 = $row->{val} if ($addr eq '3e');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}
	
		$sth->finish;
	
		$dt = "N:$val1:$val2:$val3:$val4";
		&update_rrd($rrddir, "SL_$rrd", $dt, 'SL', $verbose) if ($wr);
	}
	
	
	
	# *************************************************************************
	# WS sensors
	# *************************************************************************
	
	$wr = 0;
	
	foreach my $sensor (@sensors) {
		next if ($sensor->{proto} ne 'WS');
		my $addr = $sensor->{addr};
		my $name = $sensor->{name};
		my $rrd  = $sensor->{rrd};
		my $query = "SELECT * FROM valeurs WHERE proto='WS' AND addr='$addr'";
		# print "$query\n";
		my $sth = $dbh->prepare($query);
		my $res = $sth->execute;
	        
		$val1 = $val2 = $val3 = $val4 = $val5 = $val6 = $val7 = $val8 = 'U';
		while (my $row = $sth->fetchrow_hashref) {
			$val1 = $row->{val} if ($row->{type} eq 'tempe');
			$val2 = $row->{val} if ($row->{type} eq 'hygro');
			$val3 = $row->{val} if ($row->{type} eq 'barom');
			$val4 = $row->{val} if ($row->{type} eq 'speed');
			$val5 = $row->{val} if ($row->{type} eq 'direc');
			$val6 = $row->{val} if ($row->{type} eq 'pluvi');
			$val7 = $row->{val} if ($row->{type} eq 'expos');
			$val8 = $row->{val} if ($row->{type} eq 'sunsh');
			$wr = 1;
			# print "=> ".$row->{val}." ". $row->{type}."\n";
		}
	
		$sth->finish;
	
		$dt = "N:$val1:$val2:$val3:$val4:$val5:$val6:$val7:$val8";
		&update_rrd($rrddir, "WS_$rrd", $dt, 'WS', $verbose) if ($wr);
	}
	
	
	# *************************************************************************
	# End sensors
	# *************************************************************************
	
	$dbh->disconnect;
}

sub create_rrd {
        my $rrd_file = shift;
        my $type = shift;

	print "create_rrd $rrd_file type=$type\n";

	if ($type eq 'TX') {
		RRDs::create(
			$rrd_file,
			"DS:V1:GAUGE:600:U:U",
			"DS:V2:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:600",
			"RRA:AVERAGE:0.5:6:700",
			"RRA:AVERAGE:0.5:24:775",
			"RRA:AVERAGE:0.5:288:797"
		);
	}

	elsif ($type eq 'AC') {
		RRDs::create(
			$rrd_file,
			"DS:V1:GAUGE:600:U:U",
			"DS:V2:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:600",
			"RRA:AVERAGE:0.5:6:700",
			"RRA:AVERAGE:0.5:24:775",
			"RRA:AVERAGE:0.5:288:797"
		);
	}

	elsif ($type eq 'EL') {
		RRDs::create(
			$rrd_file,
			"DS:V1:GAUGE:600:U:U",
			"DS:V2:GAUGE:600:U:U",
			"DS:V3:GAUGE:600:U:U",
			"DS:V4:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:600",
			"RRA:AVERAGE:0.5:6:700",
			"RRA:AVERAGE:0.5:24:775",
			"RRA:AVERAGE:0.5:288:797"
		);
	}

	elsif ($type eq 'VT') {
		RRDs::create(
			$rrd_file,
			"DS:V1:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:600",
			"RRA:AVERAGE:0.5:6:700",
			"RRA:AVERAGE:0.5:24:775",
			"RRA:AVERAGE:0.5:288:797"
		);
	}

	elsif ($type eq 'WS') {
		RRDs::create(
			$rrd_file,
			"DS:V1:GAUGE:600:U:U",
			"DS:V2:GAUGE:600:U:U",
			"DS:V3:GAUGE:600:U:U",
			"DS:V4:GAUGE:600:U:U",
			"DS:V5:GAUGE:600:U:U",
			"DS:V6:GAUGE:600:U:U",
			"DS:V7:GAUGE:600:U:U",
			"DS:V8:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:600",
			"RRA:AVERAGE:0.5:6:700",
			"RRA:AVERAGE:0.5:24:775",
			"RRA:AVERAGE:0.5:288:797"
		);
	}

}

sub update_rrd {
	my $rrddir = shift;
        my $rrd = shift;
        my $data = shift;
        my $type = shift;
        my $verbose = shift;

	print "RRD:$rrd -> $data\n" if ($verbose);

        my $rrd_file = "$rrddir/$rrd.rrd";

	create_rrd($rrd_file, $type) if (! -f $rrd_file);

	RRDs::update($rrd_file, $data);

        my $err = RRDs::error;
        print "ERROR unable to create '$rrd_file': $err\n" if $err;
}
END {
#	print "PlugIns postWrite unloaded\n";
}

1;  # so the require or use succeeds

