package writeGraphs;

use strict;
use DBI;
use RRDs;
use HTML::Entities;
use POSIX qw( strftime );
use constant TIMEFORMAT => '%d-%b-%y %T';

BEGIN {
	print "PlugIns writeGraphs loaded - Write HTML and graphes\n";
}

sub new {
	our $host;
	our $database;
	our $login;
	our $password;
	our $title;
	our $city;
	our $head;
	our $rrddir;
	our $graphdir;
	our $p_alt;
	our $p_off;

	do 'domo.conf';

	$p_off += 200;
	my $p_fct = 1/((1-($p_alt/44330))**5.255);
	
	my %rec;
        my $self  = {
                _verbose => 0,
                _data => \%rec,
                _host => $host,
                _database => $database,
                _login => $login,
                _password =>$password,
                _title =>$title,
                _city =>$city,
                _head =>$head,
                _rrddir =>$rrddir,
                _graphdir =>$graphdir,
                _p_off =>$p_off,
                _p_fct =>$p_fct
        };

        if (! -d $graphdir) {
                mkdir($graphdir);
                print "Directory $graphdir does not exist... Created\n";
        }

	bless($self);
	return $self;
}

sub name {
        return "writeGraphs";
}

sub verbose {
	my ( $self, $verbose ) = @_;
	$self->{_verbose} = $verbose if defined($verbose);
	return $self->{_verbose};
}

sub process {
	my ($self, $timeformat, $min, @data) = @_;

	my $t=`date -d "today 00:00" +%s`;
	chomp($t);

        my $host = $self->{_host};
        my $database = $self->{_database};
        my $login = $self->{_login};
        my $password = $self->{_password};
        my $rrddir = $self->{_rrddir};
        my $graphdir = $self->{_graphdir};
        my $verbose = $self->{_verbose};
        my $title = $self->{_title};
        my $city = $self->{_city};
        my $head = $self->{_head};
        my $p_off = $self->{_p_off};
        my $p_fct = $self->{_p_fct};

	print "writeGraphs process\n" if ($verbose);
	print "p_off=$p_off p_fct=$p_fct\n" if ($verbose);

	my $idx = 0;
	my $dbh = DBI->connect( "DBI:mysql:database=$database;host=$host",
	                        "$login",
	                        "$password",
	                        {'RaiseError' => 1}
	                );
	
	# Max number of lines per graph
	my $maxlines = 16;
	
	my $query;
	my $rec;
	my $sth;
	my $res;
	
	$query = "SELECT * FROM graphes ORDER BY id";
	$sth = $dbh->prepare($query);
	$res = $sth->execute;
	my @graphes = ();
	while ($rec = $sth->fetchrow_hashref) {
		push @graphes, $rec;
	}
	$sth->finish;
	
	$query = "SELECT * FROM courbes";
	$sth = $dbh->prepare($query);
	$res = $sth->execute;
	my @courbes = ();
	while ($rec = $sth->fetchrow_hashref) {
		push @courbes, $rec;
	}
	$sth->finish;
	
	$query = "SELECT * FROM names";
	$sth = $dbh->prepare($query);
	$res = $sth->execute;
	my @names = ();
	while ($rec = $sth->fetchrow_hashref) {
		push @names, $rec;
	}
	$sth->finish;
	
	my $str = '';
	my $pos = 1;
	my $tr = 0;
	
	foreach my $graphe (@graphes) {
		# my @lines = ($graphe->{line1},$graphe->{line2},$graphe->{line3},$graphe->{line4},$graphe->{line5},$graphe->{line6});
		my @lines = split(':',$graphe->{curves});
		my $graph_file = "graph_".$pos++;
		my $graph_title = $graphe->{title};
		my $name = $graphe->{name};
		my $unit = $graphe->{unit};
		my $side = $graphe->{side};
		$unit =~ s/%/%%/g;
		my $format = $graphe->{format};
	
		if ($name eq '__SEP__') {
			if ($tr == 1 && $side == 0) {
				$str .= "  echo \"</tr>\\n\";\n";
				$tr = 0;
			}
			if ($tr == 0 || ($tr == 1 && $side == 0)) {
				$str .= "  echo \"<tr>\";\n";
				$tr = 1;
			}
			$str .= "  echo \"<td colspan=2><h2><br>".encode_entities($graph_title, "\200-\377")."</h2><hr></td>\";\n";
		}
		else {
	
			my @rrd_name;
			my @rrd_file;
			my @rrd_mult;
			my @graph_name;
			my @valid;

			# foreach my $line (@lines) {
			for (my $i = 0 ; $i < $maxlines ; $i++) {
				my $courbe = 0;
				foreach (@courbes) {
					$courbe = $_ if ($lines[$i] && $_->{id} == $lines[$i]);
				}

				if ($courbe && $courbe->{sensor} > 0) {
					$rrd_mult[$i] = $courbe->{multiply};
					my $type = $courbe->{type};
		
					my $name = 0;
					foreach (@names) {
						$name = $_ if ($_->{id} == $courbe->{sensor});
					}
					next if ($name == 0);
		
					my $rrdf = $name->{proto};
					if ($rrdf eq 'AC') {
						$rrd_name[$i] = 'V1' if ($type eq 'inten');
						$rrd_name[$i] = 'V2' if ($type eq 'power');
						$rrdf = 'AC';
						$rrd_file[$i] = $rrdf . '_' . $name->{rrd};
						$graph_name[$i] = $name->{name};
						$valid[$i] = 1;
					}
					elsif ($rrdf eq 'VT') {
						$rrd_name[$i] = 'V1' if ($type eq 'volta');
						$rrdf = 'VT';
						$rrd_file[$i] = $rrdf . '_' . $name->{rrd};
						$graph_name[$i] = $name->{name};
						$valid[$i] = 1;
					}
					elsif ($rrdf eq 'EL') {
						my $nb = $name->{rrd};
						$nb = 4 if ($type eq 'volta');
						$rrd_name[$i] = 'V'.$nb;
						$rrd_file[$i] = $rrdf . '_1';
						$graph_name[$i] = $name->{name};
						$valid[$i] = 1;
					}
					elsif ($rrdf eq 'TX') {
						$rrd_name[$i] = 'V1' if ($type eq 'tempe');
						$rrd_name[$i] = 'V2' if ($type eq 'hygro');
						$rrd_file[$i] = $rrdf . '_' . $name->{rrd};
						$graph_name[$i] = $name->{name};
						$valid[$i] = 1;
					}
					elsif ($rrdf eq 'WS') {
						$rrd_name[$i] = 'V1' if ($type eq 'tempe');
						$rrd_name[$i] = 'V2' if ($type eq 'hygro');
						$rrd_name[$i] = 'V3' if ($type eq 'barom');
						$rrd_name[$i] = 'V4' if ($type eq 'speed');
						$rrd_name[$i] = 'V5' if ($type eq 'direc');
						$rrd_name[$i] = 'V6' if ($type eq 'pluvi');
						$rrd_name[$i] = 'V7' if ($type eq 'expos');
						$rrd_name[$i] = 'V8' if ($type eq 'sunsh');
						$rrd_file[$i] = $rrdf . '_' . $name->{rrd};
						$graph_name[$i] = $name->{name};
						$valid[$i] = 1;
						$rrd_mult[$i] = "$p_off,+,$p_fct,*" if ($type eq 'barom');
					}
				}
				else {
					$rrd_name[$i] = 'V1';
					$rrd_file[$i] = 'TX_T';
					$graph_name[$i] = '';
					$rrd_mult[$i] = 0;
					$valid[$i] = 0;
				}
			}
	
			&draw_graph_norm($rrddir, $graphdir, $maxlines, \@rrd_file, \@rrd_name, $graph_file, $name." ".$graphe->{unit}, $unit, \@graph_name, $format, $format, \@valid, "P0", 0, \@rrd_mult, $t);
		
			if ($tr == 1 && $side == 0) {
				$str .= "  echo \"</tr>\\n\";\n";
				$tr = 0;
			}
			if ($tr == 0 || ($tr == 1 && $side == 0)) {
				$str .= "  echo \"<tr>\";\n";
				$tr = 1;
			}
			$str .= "  element(\"$graph_file\", \"".encode_entities($graph_title, "\200-\377")."\");\n";
		}
	}
	
	if ($tr == 1) {
		$str .= "  echo \"</tr>\\n\";\n";
		$tr = 0;
	}
	
	$dbh->disconnect;
	
	&create_index($graphdir, $str, $title, $head, $city);

	my $date = strftime("%x %R", localtime(time+10));
	my $flag = $graphdir.'/date.flg';
	open FILE, ">$flag" or print "'Create $flag': $!";;
	print FILE $date;
	close FILE;
}

sub create_index {
	my $graphdir = shift;
	my $gr = shift;
	my $title = shift;
	my $head = shift;
	my $city = shift;

	my $template = $graphdir.'/index.template';
	my $index = $graphdir.'/index.php';

	my $t = encode_entities($title, "\200-\377");
	$t =~ s/__DATE__/__date__/g;
	my $str = "function def() {\n  global \$date;\n  echo \"<script>setTitle('$t');</script>\n\";\n  echo \"$head\\n<center><table padding='0' cellspacing='5'>\\n\";\n";
	$str .= $gr;
	$str .= "  echo \"</table>\\n\";\n}\n";

	open FILE, $template or print "'$template': $!";;
	my $file = do { local $/; <FILE> };
	close FILE;

	$t = "echo \"".encode_entities($title, "\200-\377")."\"";
	my $d = "<span id=\\\"date\\\">\$date</span>";
	$file =~ s/__DEF__/$str/g;
	$file =~ s/__DATE__/$d/g;
	$file =~ s/__TITLE__/$t/g;
	$file =~ s/__CITY__/$city/g;

	open FILE, ">$index" or print "'Create $index': $!";;
	print FILE $file;
	close FILE;
}

sub draw_graph_norm
{
	my $rrddir = shift;
	my $graphdir = shift;
	my $maxlines = shift;
	my $rrd = shift;
	my $n = shift;
	my $png = shift;
	my $label = shift;
	my $unit = shift;
	my $p = shift;
	my $format = shift;
	my $fmm = shift;
	my $ok = shift;
	my $chauf = shift;
	my $back = shift;
	my $f = shift;
	my $t = shift;
	my $u = $t - 86400;

	my @c =( "C00000", "00C000", "0000C0", "C000C0", "C0C000", "000000");

	my $img = "PNG";              # GIF or PNG?
	
	# For consistency, the extension should be whatever you set $img to.
	my $daily = "day.png";     # Name of daily graph
	my $weekly = "week.png";   # Name of weekly graph
	my $monthly = "month.png"; # Name of monthly graph
	my $yearly = "year.png";   # Name of yearly graph
	
	my @sp;
	for (my $i = 0 ; $i < $maxlines ; $i++) {
		$sp[$i] = sprintf("%-15s",@$p[$i]);
	}

	my @args;
	my @args_j;
	for (my $i = 0 ; $i < $maxlines ; $i++) {
		if (@$ok[$i]) {
			push @args_j, (
                		"DEF:V$i=$rrddir/@$rrd[$i].rrd:@$n[$i]:AVERAGE:start=end-29h",
                		"CDEF:mytemp$i=V$i,@$f[$i]",
				"LINE1:mytemp$i#$c[$i]:$sp[$i]",
				"GPRINT:mytemp$i:LAST:$format $unit ",
				"GPRINT:mytemp$i:MIN:Min\\:$fmm $unit", 
				"GPRINT:mytemp$i:MAX:Max\\:$fmm $unit\\l" ,
			);
			push @args, (
                		"DEF:V$i=$rrddir/@$rrd[$i].rrd:@$n[$i]:AVERAGE",
                		"CDEF:mytemp$i=V$i,@$f[$i]",
				"LINE1:mytemp$i#$c[$i]:$sp[$i]",
				"GPRINT:mytemp$i:LAST:$format $unit ",
				"GPRINT:mytemp$i:MIN:Min\\:$fmm $unit", 
				"GPRINT:mytemp$i:MAX:Max\\:$fmm $unit\\l" ,
			);
		}
	}

	RRDs::graph "$graphdir/$png-$daily",
		"--start", "-100800",
		"--vertical-label","$label",
		"--imgformat","$img",
		"--alt-autoscale",
		"--units-exponent", "0",
		"--alt-y-grid",
		"VRULE:$t#ff0000",
		"VRULE:$u#ff0000",
		@args_j;

	my $ERR=RRDs::error;
	print "ERROR while drawing $graphdir/$png-$daily: $ERR\n" if $ERR;

	RRDs::graph "$graphdir/$png-$weekly",
		# "--lazy",
		"--start", "-604800",
		"--vertical-label","$label",
		"--imgformat","$img",
		"--alt-autoscale",
		"--units-exponent", "0",
		"--alt-y-grid",
		@args;

	$ERR=RRDs::error;
	print "ERROR while drawing $graphdir/$png-$daily: $ERR\n" if $ERR;

	RRDs::graph "$graphdir/$png-$monthly",
		# "--lazy",
		"--start", "-2600640",
		"--vertical-label","$label",
		"--imgformat","$img",
		"--alt-autoscale",
		"--units-exponent", "0",
		"--alt-y-grid",
		@args;

	$ERR=RRDs::error;
	print "ERROR while drawing $graphdir/$png-$daily: $ERR\n" if $ERR;

	RRDs::graph "$graphdir/$png-$yearly",
		# "--lazy",
		"--start", "-31557600",
		"--vertical-label","$label",
		"--imgformat","$img",
		"--alt-autoscale",
		"--units-exponent", "0",
		"--alt-y-grid",
		@args;

	$ERR=RRDs::error;
	print "ERROR while drawing $graphdir/$png-$daily: $ERR\n" if $ERR;
}


END {
#	print "PlugIns writeGraphs unloaded\n";
}

1;  # so the require or use succeeds

